#include <Servo.h>
#include "SR04.h"
// RF24Network - Version: 1.0.8
#include <RF24Network.h>
#include <RF24Network_config.h>

// RF24 - Version: 1.3.1
#include <RF24.h>
#include "RF24_config.h"
#include <nRF24L01.h>

RF24 radio(7,8);                    // nRF24L01(+) radio attached using Getting Started board 
RF24Network network(radio);          // Network uses that radio

struct positioning {
  int angle;    // variable to store the servo position
  int distance;
  int vehicle;
  int sensorid;
};

#define TRIG_PIN 4
#define ECHO_PIN 5
#define SERVO_PIN 9
#define max_number_objects 10

const uint16_t this_node = 03;        // Address of our node in Octal format (Node01,Node02, ...)
const uint16_t other_node = 00;       // Address of the master node (Node00) in Octal format
#define min_angle 0
#define max_angle 100

SR04 sr04 = SR04(ECHO_PIN,TRIG_PIN);
int distance;
int valid_max_distance = 80;
int valid_min_distance = 1;

Servo myservo;  // create servo object to control a servo
// twelve servo objects can be created on most boards

int angle = 0;    // variable to store the servo position
int sensorid = 1;
int vehicle = 0;
int last_distance = 0;
int last_angle_right = 0;
int last_angle_left = 0;
const int accuracy_distance = 2; // cm
const int accuracy_angle = 2; // º
boolean pos_sent = false;
String strdetections;
char strtosend[50];
boolean recognized_by_master = false;


RF24NetworkHeader header;

int calculateDistance(){ 
  long duration;
  
  digitalWrite(TRIG_PIN, LOW); 
  delayMicroseconds(2);
  // Sets the trigPin on HIGH state for 10 micro seconds
  digitalWrite(TRIG_PIN, HIGH); 
  delayMicroseconds(10);
  digitalWrite(TRIG_PIN, LOW);
  duration = pulseIn(ECHO_PIN, HIGH); // Reads the echoPin, returns the sound wave travel time in microseconds
  return duration*0.034/2;
}
void setup() {
  Serial.begin(9600);
  
  radio.begin();
  radio.setPALevel(RF24_PA_MIN);
  radio.setDataRate(RF24_250KBPS);
  network.begin(70,this_node);
  
  myservo.attach(SERVO_PIN);  // attaches the servo on pin 9 to the servo object
  strdetections = "";
  strdetections = strdetections + this_node + "#";
}

void loop() {
  
  if (not recognized_by_master) {
    RF24NetworkHeader header(other_node);
    strdetections = "";
    strdetections = strdetections + this_node + "#RECOGNITION";
    strdetections.toCharArray(strtosend,50);
    bool message_sent = network.write(header,&strtosend,sizeof(strtosend));
    if (message_sent) recognized_by_master = true;
    strdetections = "";
    strdetections = strdetections + this_node + "#";
  }
  
  unsigned int next_object_to_send = 0;
  network.update();                  // Check the network regularly
  while ( network.available() ) {     // Is there anything ready for us?
    network.read(header, &next_object_to_send, sizeof(next_object_to_send));
  }  
  if (not (next_object_to_send == this_node)) return;
  
  last_angle_right = 0;
  last_distance = 0;
  pos_sent = false;
  for (angle = min_angle; angle <= max_angle; angle += 2) { // goes from 0 degrees to 180 degrees
    // in steps of 1 degree
    myservo.write(angle);              // tell servo to go to position in variable 'angle'
    delay(20);                       // waits 15ms for the servo to reach the position
    distance = calculateDistance();
    if ((distance < valid_max_distance) and (distance > valid_min_distance)) {
      if ((abs(distance - last_distance) <= accuracy_distance) or (abs(angle - last_angle_right) <= accuracy_angle)) {
                
        if (not pos_sent) {
          pos_sent = true;
          strdetections = strdetections + distance + "," + angle + ";";
        }
      } 
      last_angle_right = angle;
      last_distance = distance;
    } else {
        last_angle_right = 0;
        last_distance = 0;
        pos_sent = false;
    }
  }

  if (strdetections.length() > 2) {
    strdetections = strdetections + ":";
    strdetections.toCharArray(strtosend,50);
    RF24NetworkHeader header(other_node);
    bool message_sent = network.write(header,&strtosend,sizeof(strtosend));
    Serial.println (strdetections);
    strdetections = "";
    strdetections = strdetections + this_node + "#";
  }
  for (angle = max_angle; angle >= min_angle; angle -= 10) { // goes from 180 degrees to 0 degrees
    // in steps of 1 degree
    myservo.write(angle);              // tell servo to go to position in variable 'angle'
    delay(50);                       // waits 15ms for the servo to reach the position
  }
}
